@@ -19,10 +19,10 @@ import (
 	"strconv"
 	"strings"
 
-	"golang.org/x/crypto/ssh/terminal"
-
 	"github.com/gorilla/websocket"
 	"github.com/gosexy/gettext"
+	"golang.org/x/crypto/ssh/terminal"
+
 	"github.com/lxc/lxd/shared"
 )
 
@@ -520,7 +520,7 @@ func (c *Client) ListContainers() ([]string, error) {
 	return names, nil
 }
 
-func (c *Client) CopyImage(image string, dest *Client, copy_aliases bool, aliases []string) error {
+func (c *Client) CopyImage(image string, dest *Client, copy_aliases bool, aliases []string, public bool) error {
 	uri := c.url(shared.APIVersion, "images", image, "export")
 	raw, err := c.getRaw(uri)
 
@@ -541,14 +541,18 @@ func (c *Client) CopyImage(image string, dest *Client, copy_aliases bool, aliase
 	}
 	postreq.Header.Set("User-Agent", shared.UserAgent)
 	postreq.Header.Set("X-LXD-filename", cd[1])
-	postreq.Header.Set("X-LXD-public", "0")
+	if public {
+		postreq.Header.Set("X-LXD-public", "1")
+	} else {
+		postreq.Header.Set("X-LXD-public", "0")
+	}
 	imgProps := url.Values{}
 	for key, value := range info.Properties {
 		imgProps.Set(key, value)
 	}
 	postreq.Header.Set("X-LXD-properties", imgProps.Encode())
 
-	postresp, err := c.http.Do(postreq)
+	postresp, err := dest.http.Do(postreq)
 	if err != nil {
 		return err
 	}
@@ -650,18 +654,18 @@ func (c *Client) ExportImage(image string, target string) (*Response, string, er
 
 }
 
-func (c *Client) PostImage(path string, properties []string, public bool) (*Response, error) {
+func (c *Client) PostImage(path string, properties []string, public bool, aliases []string) (string, error) {
 	uri := c.url(shared.APIVersion, "images")
 
 	f, err := os.Open(path)
 	if err != nil {
-		return nil, err
+		return "", err
 	}
 	defer f.Close()
 
 	req, err := http.NewRequest("POST", uri, f)
 	if err != nil {
-		return nil, err
+		return "", err
 	}
 	req.Header.Set("User-Agent", shared.UserAgent)
 
@@ -683,7 +687,7 @@ func (c *Client) PostImage(path string, properties []string, public bool) (*Resp
 			if eqIndex > -1 {
 				imgProps.Set(value[:eqIndex], value[eqIndex+1:])
 			} else {
-				return nil, fmt.Errorf(gettext.Gettext("Bad image property: %s\n"), value)
+				return "", fmt.Errorf(gettext.Gettext("Bad image property: %s\n"), value)
 			}
 
 		}
@@ -693,10 +697,34 @@ func (c *Client) PostImage(path string, properties []string, public bool) (*Resp
 
 	raw, err := c.http.Do(req)
 	if err != nil {
-		return nil, err
+		return "", err
 	}
 
-	return HoistResponse(raw, Sync)
+	resp, err := HoistResponse(raw, Sync)
+	if err != nil {
+		return "", err
+	}
+
+	jmap, err := resp.MetadataAsMap()
+	if err != nil {
+		return "", err
+	}
+
+	fingerprint, err := jmap.GetString("fingerprint")
+	if err != nil {
+		return "", err
+	}
+
+	/* add new aliases */
+	for _, alias := range aliases {
+		c.DeleteAlias(alias)
+		err = c.PostAlias(alias, alias, fingerprint)
+		if err != nil {
+			fmt.Printf(gettext.Gettext("Error adding alias %s\n"), alias)
+		}
+	}
+
+	return fingerprint, nil
 }
 
 func (c *Client) GetImageInfo(image string) (*shared.ImageInfo, error) {
@@ -719,13 +747,13 @@ func (c *Client) PutImageProperties(name string, p shared.ImageProperties) error
 	return err
 }
 
-func (c *Client) ListImages() ([]string, error) {
-	resp, err := c.get("images")
+func (c *Client) ListImages() ([]shared.ImageInfo, error) {
+	resp, err := c.get("images?recursion=1")
 	if err != nil {
 		return nil, err
 	}
 
-	var result []string
+	var result []shared.ImageInfo
 	if err := json.Unmarshal(resp.Metadata, &result); err != nil {
 		return nil, err
 	}
@@ -919,6 +947,20 @@ func (c *Client) Init(name string, imgremote string, image string, profiles *[]s
 	return c.post("containers", body, Async)
 }
 
+func (c *Client) LocalCopy(source string, name string, config map[string]string, profiles []string) (*Response, error) {
+	body := shared.Jmap{
+		"source": shared.Jmap{
+			"type":   "copy",
+			"source": source,
+		},
+		"name":     name,
+		"config":   config,
+		"profiles": profiles,
+	}
+
+	return c.post("containers", body, Async)
+}
+
 type execMd struct {
 	FDs map[string]string `json:"fds"`
 }
