@@ -2,15 +2,21 @@ package main
 
 import (
 	"database/sql"
+	"encoding/hex"
 	"errors"
 	"fmt"
+	"os"
 	"time"
 
 	"github.com/lxc/lxd/shared"
 	_ "github.com/stgraber/lxd-go-sqlite3"
 )
 
-const DB_CURRENT_VERSION int = 4
+const DB_CURRENT_VERSION int = 5
+
+var (
+	DbErrAlreadyDefined = fmt.Errorf("already exists")
+)
 
 func createDb(p string) (*sql.DB, error) {
 	db, err := sql.Open("sqlite3", p)
@@ -26,6 +32,12 @@ CREATE TABLE certificates (
     certificate TEXT NOT NULL,
     UNIQUE (fingerprint)
 );
+CREATE TABLE config (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    key VARCHAR(255) NOT NULL,
+    value TEXT,
+    UNIQUE (key)
+);
 CREATE TABLE containers (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
     name VARCHAR(255) NOT NULL,
@@ -155,6 +167,54 @@ func getSchema(db *sql.DB) (int, error) {
 	return 0, nil
 }
 
+func updateFromV4(db *sql.DB) error {
+	stmt := `
+CREATE TABLE config (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    key VARCHAR(255) NOT NULL,
+    value TEXT,
+    UNIQUE (key)
+);
+INSERT INTO schema (version, updated_at) VALUES (?, strftime("%s"));`
+	_, err := db.Exec(stmt, 5)
+	if err != nil {
+		return err
+	}
+
+	passfname := shared.VarPath("adminpwd")
+	passOut, err := os.Open(passfname)
+	old_password := ""
+	if err == nil {
+		defer passOut.Close()
+		buff := make([]byte, 96)
+		_, err = passOut.Read(buff)
+		if err != nil {
+			return err
+		}
+
+		old_password = hex.EncodeToString(buff)
+		stmt := `INSERT INTO config (key, value) VALUES ("core.trust_password", ?);`
+
+		_, err := db.Exec(stmt, old_password)
+		if err != nil {
+			return err
+		}
+
+		return os.Remove(passfname)
+	}
+
+	return nil
+}
+
+func updateFromV3(db *sql.DB) error {
+	err := createDefaultProfile(db)
+	if err != nil {
+		return err
+	}
+	_, err = db.Exec(`INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`, 4)
+	return err
+}
+
 func updateFromV2(db *sql.DB) error {
 	stmt := `
 CREATE TABLE containers_devices (
@@ -216,15 +276,6 @@ INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`
 	return err
 }
 
-func updateFromV3(db *sql.DB) error {
-	err := createDefaultProfile(db)
-	if err != nil {
-		return err
-	}
-	_, err = db.Exec(`INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`, 4)
-	return err
-}
-
 /* Yeah we can do htis in a more clever way */
 func updateFromV1(db *sql.DB) error {
 	// v1..v2 adds images aliases
@@ -288,6 +339,12 @@ func updateDb(db *sql.DB, prev_version int) error {
 			return err
 		}
 	}
+	if prev_version < 5 {
+		err = updateFromV4(db)
+		if err != nil {
+			return err
+		}
+	}
 	return nil
 }
 
