@@ -508,6 +508,10 @@ func GetTLSConfig(certf string, keyf string) (*tls.Config, error) {
 		Certificates:       []tls.Certificate{cert},
 		MinVersion:         tls.VersionTLS12,
 		MaxVersion:         tls.VersionTLS12,
+		CipherSuites: []uint16{
+			tls.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
+			tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256},
+		PreferServerCipherSuites: true,
 	}
 
 	tlsConfig.BuildNameToCertificate()
@@ -533,3 +537,28 @@ func WriteAllBuf(w io.Writer, buf *bytes.Buffer) error {
 		}
 	}
 }
+
+// CopyFile copies a file, overwriting the target if it exists.
+func CopyFile(dest string, source string) error {
+	s, err := os.Open(source)
+	if err != nil {
+		return err
+	}
+	defer s.Close()
+
+	d, err := os.Create(dest)
+	if err != nil {
+		if os.IsExist(err) {
+			d, err = os.OpenFile(dest, os.O_WRONLY, 0700)
+			if err != nil {
+				return err
+			}
+		} else {
+			return err
+		}
+	}
+	defer d.Close()
+
+	_, err = io.Copy(d, s)
+	return err
+}
